@@ -23,7 +23,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-igrp.c,v 1.15 2000/09/29 04:58:40 guy Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/tcpdump/print-igrp.c,v 1.16 2001/06/15 22:17:32 fenner Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -92,9 +92,7 @@ igrp_print(register const u_char *bp, u_int length, register const u_char *bp2)
 	hdr = (struct igrphdr *)bp;
 	ip = (struct ip *)bp2;
 	cp = (u_char *)(hdr + 1);
-        (void)printf("%s > %s: igrp: ",
-	    ipaddr_string(&ip->ip_src),
-	    ipaddr_string(&ip->ip_dst));
+        (void)printf("igrp:");
 
 	/* Header */
 	TCHECK(*hdr);
@@ -126,7 +124,7 @@ igrp_print(register const u_char *bp, u_int length, register const u_char *bp2)
 			igrp_entry_print((struct igrprte *)cp, 0, 1);
 			--next;
 		} else {
-			(void)printf("[extra bytes %d]", length);
+			(void)printf(" [extra bytes %d]", length);
 			break;
 		}
 		cp += IGRP_RTE_SIZE;
@@ -135,5 +133,5 @@ igrp_print(register const u_char *bp, u_int length, register const u_char *bp2)
 	if (nint == 0 && nsys == 0 && next == 0)
 		return;
 trunc:
-	fputs("[|igrp]", stdout);
+	fputs(" [|igrp]", stdout);
 }
