@@ -1,3 +1,25 @@
+/*
+ * Copyright: (c) 2000 United States Government as represented by the
+ *	Secretary of the Navy. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *  
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above copyright
+ *      notice, this list of conditions and the following disclaimer in
+ *      the documentation and/or other materials provided with the
+ *      distribution.
+ *   3. The names of the authors may not be used to endorse or promote
+ *      products derived from this software without specific prior
+ *      written permission.
+ *  
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ */
 /*
  * This code unmangles RX packets.  RX is the mutant form of RPC that AFS
  * uses to communicate between clients and servers.
@@ -8,12 +30,11 @@
  * Bah.  If I never look at rx_packet.h again, it will be too soon.
  *
  * Ken Hornstein <kenh@cmf.nrl.navy.mil>
- *
  */
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-rx.c,v 1.20 2001/01/10 08:12:01 fenner Exp $";
+    "@(#) $Header: /tcpdump/master/tcpdump/print-rx.c,v 1.27 2001/10/20 07:41:55 itojun Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -35,9 +56,6 @@ static const char rcsid[] =
 #include "addrtoname.h"
 #include "extract.h"
 
-#undef NOERROR					/* Solaris sucks */
-#include <arpa/nameser.h>
-
 #include "rx.h"
 
 #include "ip.h"
@@ -56,12 +74,18 @@ static struct tok rx_types[] = {
 	{ 0,				NULL },
 };
 
-static struct tok rx_flags[] = {
-	{ RX_CLIENT_INITIATED,	"client-init" },
-	{ RX_REQUEST_ACK,	"req-ack" },
-	{ RX_LAST_PACKET,	"last-pckt" },
-	{ RX_MORE_PACKETS,	"more-pckts" },
-	{ RX_FREE_PACKET,	"free-pckt" }
+static struct double_tok {
+	int flag;		/* Rx flag */
+	int packetType;		/* Packet type */
+	char *s;		/* Flag string */
+} rx_flags[] = {
+	{ RX_CLIENT_INITIATED,	0,			"client-init" },
+	{ RX_REQUEST_ACK,	0,			"req-ack" },
+	{ RX_LAST_PACKET,	0,			"last-pckt" },
+	{ RX_MORE_PACKETS,	0,			"more-pckts" },
+	{ RX_FREE_PACKET,	0,			"free-pckt" },
+	{ RX_SLOW_START_OK,	RX_PACKET_TYPE_ACK,	"slow-start" },
+	{ RX_JUMBO_PACKET,	RX_PACKET_TYPE_DATA,	"jumbogram" }
 };
 
 static struct tok fs_req[] = {
@@ -99,6 +123,7 @@ static struct tok fs_req[] = {
 	{ 161,		"dfs-lookup" },
 	{ 162,		"dfs-flushcps" },
 	{ 163,		"dfs-symlink" },
+	{ 220,		"residency" },
 	{ 0,		NULL },
 };
 
@@ -114,6 +139,10 @@ static struct tok cb_req[] = {
 	{ 212,		"whoareyou" },
 	{ 213,		"initcb3" },
 	{ 214,		"probeuuid" },
+	{ 215,		"getsrvprefs" },
+	{ 216,		"getcellservdb" },
+	{ 217,		"getlocalcell" },
+	{ 218,		"getcacheconf" },
 	{ 0,		NULL },
 };
 
@@ -139,6 +168,7 @@ static struct tok pt_req[] = {
 	{ 518,		"get-cps2" },
 	{ 519,		"get-host-cps" },
 	{ 520,		"update-entry" },
+	{ 521,		"list-entries" },
 	{ 0,		NULL },
 };
 
@@ -176,6 +206,7 @@ static struct tok vldb_req[] = {
 	{ 531,		"linked-list-u" },
 	{ 532,		"regaddr" },
 	{ 533,		"get-addrs-u" },
+	{ 534,		"list-attrib-n2" },
 	{ 0,		NULL },
 };
 
@@ -270,6 +301,8 @@ static struct tok bos_req[] = {
 	{ 112,		"start-bozo-log" },
 	{ 113,		"wait-all" },
 	{ 114,		"get-instance-strings" },
+	{ 115,		"get-restricted" },
+	{ 116,		"set-restricted" },
 	{ 0,		NULL },
 };
 
@@ -439,7 +472,9 @@ rx_print(register const u_char *bp, int length, int sport, int dport,
 
 		if (vflag > 1)
 			for (i = 0; i < NUM_RX_FLAGS; i++) {
-				if (rxh->flags & rx_flags[i].v) {
+				if (rxh->flags & rx_flags[i].flag &&
+				    (!rx_flags[i].packetType ||
+				     rxh->type == rx_flags[i].packetType)) {
 					if (!firstflag) {
 						firstflag = 1;
 						printf(" ");
@@ -635,14 +670,16 @@ rx_cache_find(const struct rx_header *rxh, const struct ip *ip, int sport,
 			printf(" fid %d/%d/%d", (int) n1, (int) n2, (int) n3); \
 		}
 
-#define STROUT(MAX) { int i; \
+#define STROUT(MAX) { unsigned int i; \
 			TCHECK2(bp[0], sizeof(int32_t)); \
-			i = (int) EXTRACT_32BITS(bp); \
+			i = EXTRACT_32BITS(bp); \
+			if (i > (MAX)) \
+				goto trunc; \
 			bp += sizeof(int32_t); \
-			TCHECK2(bp[0], i); \
-			strncpy(s, (char *) bp, min(MAX, i)); \
-			s[i] = '\0'; \
-			printf(" \"%s\"", s); \
+			printf(" \""); \
+			if (fn_printn(bp, i, snapend)) \
+				goto trunc; \
+			printf("\""); \
 			bp += ((i + sizeof(int32_t) - 1) / sizeof(int32_t)) * sizeof(int32_t); \
 		}
 
@@ -719,15 +756,20 @@ rx_cache_find(const struct rx_header *rxh, const struct ip *ip, int sport,
  */
 
 #define VECOUT(MAX) { char *sp; \
+			char s[AFSNAMEMAX]; \
 			int k; \
-			TCHECK2(bp[0], MAX * sizeof(int32_t)); \
+			if ((MAX) + 1 > sizeof(s)) \
+				goto trunc; \
+			TCHECK2(bp[0], (MAX) * sizeof(int32_t)); \
 			sp = s; \
-			for (k = 0; k < MAX; k++) { \
+			for (k = 0; k < (MAX); k++) { \
 				*sp++ = (char) EXTRACT_32BITS(bp); \
 				bp += sizeof(int32_t); \
 			} \
-			s[MAX] = '\0'; \
-			printf(" \"%s\"", s); \
+			s[(MAX)] = '\0'; \
+			printf(" \""); \
+			fn_print(s, NULL); \
+			printf("\""); \
 		}
 
 static void
@@ -808,7 +850,6 @@ fs_print(register const u_char *bp, int length)
 {
 	int fs_op;
 	unsigned long i;
-	char s[AFSNAMEMAX];
 
 	if (length <= sizeof(struct rx_header))
 		return;
@@ -959,7 +1000,6 @@ static void
 fs_reply_print(register const u_char *bp, int length, int32_t opcode)
 {
 	unsigned long i;
-	char s[AFSNAMEMAX];
 	struct rx_header *rxh;
 
 	if (length <= sizeof(struct rx_header))
@@ -1087,7 +1127,9 @@ acl_print(u_char *s, int maxsize, u_char *end)
 		if (sscanf((char *) s, "%s %d\n%n", user, &acl, &n) != 2)
 			goto finish;
 		s += n;
-		printf(" +{%s ", user);
+		printf(" +{");
+		fn_print(user, NULL);
+		printf(" ");
 		ACLOUT(acl);
 		printf("}");
 		if (s > end)
@@ -1098,7 +1140,9 @@ acl_print(u_char *s, int maxsize, u_char *end)
 		if (sscanf((char *) s, "%s %d\n%n", user, &acl, &n) != 2)
 			goto finish;
 		s += n;
-		printf(" -{%s ", user);
+		printf(" -{");
+		fn_print(user, NULL);
+		printf(" ");
 		ACLOUT(acl);
 		printf("}");
 		if (s > end)
@@ -1251,7 +1295,6 @@ static void
 prot_print(register const u_char *bp, int length)
 {
 	unsigned long i;
-	char s[AFSNAMEMAX];
 	int pt_op;
 
 	if (length <= sizeof(struct rx_header))
@@ -1395,7 +1438,6 @@ prot_reply_print(register const u_char *bp, int length, int32_t opcode)
 {
 	struct rx_header *rxh;
 	unsigned long i;
-	char s[AFSNAMEMAX];
 
 	if (length < sizeof(struct rx_header))
 		return;
@@ -1508,7 +1550,6 @@ vldb_print(register const u_char *bp, int length)
 {
 	int vldb_op;
 	unsigned long i;
-	char s[AFSNAMEMAX];
 
 	if (length <= sizeof(struct rx_header))
 		return;
@@ -1602,7 +1643,6 @@ vldb_reply_print(register const u_char *bp, int length, int32_t opcode)
 {
 	struct rx_header *rxh;
 	unsigned long i;
-	char s[AFSNAMEMAX];
 
 	if (length < sizeof(struct rx_header))
 		return;
@@ -1789,7 +1829,6 @@ static void
 kauth_print(register const u_char *bp, int length)
 {
 	int kauth_op;
-	char s[AFSNAMEMAX];
 
 	if (length <= sizeof(struct rx_header))
 		return;
@@ -2016,7 +2055,6 @@ static void
 bos_print(register const u_char *bp, int length)
 {
 	int bos_op;
-	char s[BOSNAMEMAX];
 
 	if (length <= sizeof(struct rx_header))
 		return;
